@@ -2,9 +2,14 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <math.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
+#ifdef HAVE_RSVG
+#include <librsvg/rsvg.h>
+#include <librsvg/rsvg-cairo.h>
+#endif
 
 #include "eog-marshal.h"
 #include "eog-scroll-view.h"
@@ -132,6 +137,8 @@ struct _EogScrollViewPrivate {
 
 	/* the type of the cursor we are currently showing */
 	EogScrollViewCursor cursor;
+
+	cairo_surface_t *background_surface;
 };
 
 static void scroll_by (EogScrollView *view, int xofs, int yofs);
@@ -251,35 +258,40 @@ update_scrollbar_values (EogScrollView *view)
 	EogScrollViewPrivate *priv;
 	int scaled_width, scaled_height;
 	int xofs, yofs;
-	GtkAllocation *allocation;
+	gdouble page_size,page_increment,step_increment;
+	gdouble lower, upper, value;
+	GtkAllocation allocation;
 
 	priv = view->priv;
 
-	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->hbar)) && !GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->vbar)))
+	if (!gtk_widget_get_visible (GTK_WIDGET (priv->hbar))
+	    && !gtk_widget_get_visible (GTK_WIDGET (priv->vbar)))
 		return;
 
 	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
-	allocation = &GTK_WIDGET (priv->display)->allocation;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
-	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->hbar))) {
+	if (gtk_widget_get_visible (GTK_WIDGET (priv->hbar))) {
 		/* Set scroll increments */
-		priv->hadj->page_size = MIN (scaled_width, allocation->width);
-		priv->hadj->page_increment = allocation->width / 2;
-		priv->hadj->step_increment = SCROLL_STEP_SIZE;
+		page_size = MIN (scaled_width, allocation.width);
+		page_increment = allocation.width / 2;
+		step_increment = SCROLL_STEP_SIZE;
 
 		/* Set scroll bounds and new offsets */
-		priv->hadj->lower = 0;
-		priv->hadj->upper = scaled_width;
-		xofs = CLAMP (priv->xofs, 0, priv->hadj->upper - priv->hadj->page_size);
-		if (priv->hadj->value != xofs) {
-			priv->hadj->value = xofs;
+		lower = 0;
+		upper = scaled_width;
+		xofs = CLAMP (priv->xofs, 0, upper - page_size);
+		if (gtk_adjustment_get_value (priv->hadj) != xofs) {
+			value = xofs;
 			priv->xofs = xofs;
 
 			g_signal_handlers_block_matched (
 				priv->hadj, G_SIGNAL_MATCH_DATA,
 				0, 0, NULL, NULL, view);
 
-			gtk_adjustment_changed (priv->hadj);
+			gtk_adjustment_configure (priv->hadj, value, lower,
+						  upper, step_increment,
+						  page_increment, page_size);
 
 			g_signal_handlers_unblock_matched (
 				priv->hadj, G_SIGNAL_MATCH_DATA,
@@ -287,24 +299,26 @@ update_scrollbar_values (EogScrollView *view)
 		}
 	}
 
-	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->vbar))) {
-		priv->vadj->page_size = MIN (scaled_height, allocation->height);
-		priv->vadj->page_increment = allocation->height / 2;
-		priv->vadj->step_increment = SCROLL_STEP_SIZE;
+	if (gtk_widget_get_visible (GTK_WIDGET (priv->vbar))) {
+		page_size = MIN (scaled_height, allocation.height);
+		page_increment = allocation.height / 2;
+		step_increment = SCROLL_STEP_SIZE;
 
-		priv->vadj->lower = 0;
-		priv->vadj->upper = scaled_height;
-		yofs = CLAMP (priv->yofs, 0, priv->vadj->upper - priv->vadj->page_size);
+		lower = 0;
+		upper = scaled_height;
+		yofs = CLAMP (priv->yofs, 0, upper - page_size);
 
-		if (priv->vadj->value != yofs) {
-			priv->vadj->value = yofs;
+		if (gtk_adjustment_get_value (priv->vadj) != yofs) {
+			value = yofs;
 			priv->yofs = yofs;
 
 			g_signal_handlers_block_matched (
 				priv->vadj, G_SIGNAL_MATCH_DATA,
 				0, 0, NULL, NULL, view);
 
-			gtk_adjustment_changed (priv->vadj);
+			gtk_adjustment_configure (priv->vadj, value, lower,
+						  upper, step_increment,
+						  page_increment, page_size);
 
 			g_signal_handlers_unblock_matched (
 				priv->vadj, G_SIGNAL_MATCH_DATA,
@@ -342,7 +356,7 @@ eog_scroll_view_set_cursor (EogScrollView *view, EogScrollViewCursor new_cursor)
 
 	switch (new_cursor) {
 		case EOG_SCROLL_VIEW_CURSOR_NORMAL:
-			gdk_window_set_cursor (widget->window, NULL);
+			gdk_window_set_cursor (gtk_widget_get_window (widget), NULL);
 			break;
                 case EOG_SCROLL_VIEW_CURSOR_HIDDEN:
                         cursor = eog_scroll_view_create_invisible_cursor ();
@@ -353,7 +367,7 @@ eog_scroll_view_set_cursor (EogScrollView *view, EogScrollViewCursor new_cursor)
 	}
 
 	if (cursor) {
-		gdk_window_set_cursor (widget->window, cursor);
+		gdk_window_set_cursor (gtk_widget_get_window (widget), cursor);
 		gdk_cursor_unref (cursor);
 		gdk_flush();
 	}
@@ -380,8 +394,11 @@ check_scrollbar_visibility (EogScrollView *view, GtkAllocation *alloc)
 		width = alloc->width;
 		height = alloc->height;
 	} else {
-		width = GTK_WIDGET (view)->allocation.width;
-		height = GTK_WIDGET (view)->allocation.height;
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (GTK_WIDGET (view), &allocation);
+		width = allocation.width;
+		height = allocation.height;
 	}
 
 	compute_scaled_size (view, priv->zoom, &img_width, &img_height);
@@ -417,10 +434,10 @@ check_scrollbar_visibility (EogScrollView *view, GtkAllocation *alloc)
 			hbar_visible = TRUE;
 	}
 
-	if (hbar_visible != GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->hbar)))
+	if (hbar_visible != gtk_widget_get_visible (GTK_WIDGET (priv->hbar)))
 		g_object_set (G_OBJECT (priv->hbar), "visible", hbar_visible, NULL);
 
-	if (vbar_visible != GTK_WIDGET_VISIBLE (GTK_WIDGET (priv->vbar)))
+	if (vbar_visible != gtk_widget_get_visible (GTK_WIDGET (priv->vbar)))
 		g_object_set (G_OBJECT (priv->vbar), "visible", vbar_visible, NULL);
 }
 
@@ -467,7 +484,7 @@ is_image_movable (EogScrollView *view)
 
 	priv = view->priv;
 
-	return (GTK_WIDGET_VISIBLE (priv->hbar) || GTK_WIDGET_VISIBLE (priv->vbar));
+	return (gtk_widget_get_visible (priv->hbar) || gtk_widget_get_visible (priv->vbar));
 }
 
 
@@ -530,56 +547,239 @@ paint_background (EogScrollView *view, EogIRect *r, EogIRect *rect)
 
 	eog_irect_intersect (&d, r, rect);
 	if (!eog_irect_empty (&d)) {
-		gdk_window_clear_area (GTK_WIDGET (priv->display)->window,
+		gdk_window_clear_area (gtk_widget_get_window (priv->display),
 				       d.x0, d.y0,
 				       d.x1 - d.x0, d.y1 - d.y0);
 	}
 }
 
+static void
+get_transparency_params (EogScrollView *view, int *size, guint32 *color1, guint32 *color2)
+{
+	EogScrollViewPrivate *priv;
+
+	priv = view->priv;
+
+	/* Compute transparency parameters */
+	switch (priv->transp_style) {
+	case EOG_TRANSP_BACKGROUND: {
+		GdkColor color = gtk_widget_get_style (GTK_WIDGET (priv->display))->bg[GTK_STATE_NORMAL];
+
+		*color1 = *color2 = (((color.red & 0xff00) << 8)
+				       | (color.green & 0xff00)
+				       | ((color.blue & 0xff00) >> 8));
+		break; }
+
+	case EOG_TRANSP_CHECKED:
+		*color1 = CHECK_GRAY;
+		*color2 = CHECK_LIGHT;
+		break;
+
+	case EOG_TRANSP_COLOR:
+		*color1 = *color2 = priv->transp_color;
+		break;
+
+	default:
+		g_assert_not_reached ();
+	};
+
+	*size = CHECK_MEDIUM;
+}
+
+#ifdef HAVE_RSVG
+static cairo_surface_t *
+create_background_surface (EogScrollView *view)
+{
+	int check_size;
+	guint32 check_1 = 0;
+	guint32 check_2 = 0;
+	cairo_surface_t *surface;
+	cairo_t *check_cr;
+
+	get_transparency_params (view, &check_size, &check_1, &check_2);
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, check_size * 2, check_size * 2);
+	check_cr = cairo_create (surface);
+	cairo_set_source_rgba (check_cr,
+			       ((check_1 & 0xff0000) >> 16) / 255.,
+			       ((check_1 & 0x00ff00) >> 8)  / 255.,
+			        (check_1 & 0x0000ff)        / 255.,
+				1.);
+	cairo_rectangle (check_cr, 0., 0., check_size, check_size);
+	cairo_fill (check_cr);
+	cairo_translate (check_cr, check_size, check_size);
+	cairo_rectangle (check_cr, 0., 0., check_size, check_size);
+	cairo_fill (check_cr);
+
+	cairo_set_source_rgba (check_cr,
+			       ((check_2 & 0xff0000) >> 16) / 255.,
+			       ((check_2 & 0x00ff00) >> 8)  / 255.,
+			        (check_2 & 0x0000ff)        / 255.,
+				1.);
+	cairo_translate (check_cr, -check_size, 0);
+	cairo_rectangle (check_cr, 0., 0., check_size, check_size);
+	cairo_fill (check_cr);
+	cairo_translate (check_cr, check_size, -check_size);
+	cairo_rectangle (check_cr, 0., 0., check_size, check_size);
+	cairo_fill (check_cr);
+	cairo_destroy (check_cr);
+
+	return surface;
+}
+
+static void
+draw_svg_background (EogScrollView *view, cairo_t *cr, EogIRect *render_rect, EogIRect *image_rect)
+{
+	EogScrollViewPrivate *priv;
+
+	priv = view->priv;
+
+	if (priv->background_surface == NULL)
+		priv->background_surface = create_background_surface (view);
+
+	cairo_set_source_surface (cr, priv->background_surface,
+				  - (render_rect->x0 - image_rect->x0) % (CHECK_MEDIUM * 2),
+				  - (render_rect->y0 - image_rect->y0) % (CHECK_MEDIUM * 2));
+	cairo_pattern_set_extend (cairo_get_source (cr), CAIRO_EXTEND_REPEAT);
+	cairo_rectangle (cr,
+			 0,
+			 0,
+			 render_rect->x1 - render_rect->x0,
+			 render_rect->y1 - render_rect->y0);
+	cairo_fill (cr);
+}
+
+static cairo_surface_t *
+draw_svg_on_image_surface (EogScrollView *view, EogIRect *render_rect, EogIRect *image_rect)
+{
+	EogScrollViewPrivate *priv;
+	cairo_t *cr;
+	cairo_surface_t *surface;
+	cairo_matrix_t matrix, translate, scale;
+	EogTransform *transform;
+
+	priv = view->priv;
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					      render_rect->x1 - render_rect->x0,
+					      render_rect->y1 - render_rect->y0);
+	cr = cairo_create (surface);
+
+	cairo_save (cr);
+	draw_svg_background (view, cr, render_rect, image_rect);
+	cairo_restore (cr);
+
+	cairo_matrix_init_identity (&matrix);
+	transform = eog_image_get_transform (priv->image);
+	if (transform) {
+		cairo_matrix_t affine;
+		double image_offset_x = 0., image_offset_y = 0.;
+
+		eog_transform_get_affine (transform, &affine);
+		cairo_matrix_multiply (&matrix, &affine, &matrix);
+
+		switch (eog_transform_get_transform_type (transform)) {
+		case EOG_TRANSFORM_ROT_90:
+		case EOG_TRANSFORM_FLIP_HORIZONTAL:
+			image_offset_x = (double) gdk_pixbuf_get_width (priv->pixbuf);
+			break;
+		case EOG_TRANSFORM_ROT_270:
+		case EOG_TRANSFORM_FLIP_VERTICAL:
+			image_offset_y = (double) gdk_pixbuf_get_height (priv->pixbuf);
+			break;
+		case EOG_TRANSFORM_ROT_180:
+		case EOG_TRANSFORM_TRANSPOSE:
+		case EOG_TRANSFORM_TRANSVERSE:
+			image_offset_x = (double) gdk_pixbuf_get_width (priv->pixbuf);
+			image_offset_y = (double) gdk_pixbuf_get_height (priv->pixbuf);
+			break;
+		case EOG_TRANSFORM_NONE:
+		default:
+			break;
+		}
+
+		cairo_matrix_init_translate (&translate, image_offset_x, image_offset_y);
+		cairo_matrix_multiply (&matrix, &matrix, &translate);
+	}
+
+	cairo_matrix_init_scale (&scale, priv->zoom, priv->zoom);
+	cairo_matrix_multiply (&matrix, &matrix, &scale);
+	cairo_matrix_init_translate (&translate, image_rect->x0, image_rect->y0);
+	cairo_matrix_multiply (&matrix, &matrix, &translate);
+	cairo_matrix_init_translate (&translate, -render_rect->x0, -render_rect->y0);
+	cairo_matrix_multiply (&matrix, &matrix, &translate);
+
+	cairo_set_matrix (cr, &matrix);
+
+	rsvg_handle_render_cairo (eog_image_get_svg (priv->image), cr);
+	cairo_destroy (cr);
+
+	return surface;
+}
+
+static void
+draw_svg (EogScrollView *view, EogIRect *render_rect, EogIRect *image_rect)
+{
+	EogScrollViewPrivate *priv;
+	cairo_t *cr;
+	cairo_surface_t *surface;
+	GdkWindow *window;
+
+	priv = view->priv;
+
+	window = gtk_widget_get_window (GTK_WIDGET (priv->display));
+	surface = draw_svg_on_image_surface (view, render_rect, image_rect);
+
+	cr = gdk_cairo_create (window);
+	cairo_set_source_surface (cr, surface, render_rect->x0, render_rect->y0);
+	cairo_paint (cr);
+	cairo_destroy (cr);
+}
+#endif
+
 /* Paints a rectangle of the dirty region */
 static void
 paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 {
 	EogScrollViewPrivate *priv;
+	GdkPixbuf *tmp;
+	char *str;
+	GtkAllocation allocation;
 	int scaled_width, scaled_height;
-	int width, height;
 	int xofs, yofs;
 	EogIRect r, d;
-	GdkPixbuf *tmp;
 	int check_size;
-	char *str;
 	guint32 check_1 = 0;
 	guint32 check_2 = 0;
 
 	priv = view->priv;
 
-	if (!GTK_WIDGET_DRAWABLE (priv->display))
+	if (!gtk_widget_is_drawable (priv->display))
 		return;
 
 	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	if (scaled_width < 1 || scaled_height < 1)
 	{
 		r.x0 = 0;
 		r.y0 = 0;
-		r.x1 = width;
-		r.y1 = height;
+		r.x1 = allocation.width;
+		r.y1 = allocation.height;
 		paint_background (view, &r, rect);
 		return;
 	}
 
 	/* Compute image offsets with respect to the window */
 
-	if (scaled_width <= width)
-		xofs = (width - scaled_width) / 2;
+	if (scaled_width <= allocation.width)
+		xofs = (allocation.width - scaled_width) / 2;
 	else
 		xofs = -priv->xofs;
 
-	if (scaled_height <= height)
-		yofs = (height - scaled_height) / 2;
+	if (scaled_height <= allocation.height)
+		yofs = (allocation.height - scaled_height) / 2;
 	else
 		yofs = -priv->yofs;
 
@@ -592,7 +792,7 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (yofs > 0) {
 		r.x0 = 0;
 		r.y0 = 0;
-		r.x1 = width;
+		r.x1 = allocation.width;
 		r.y1 = yofs;
 		paint_background (view, &r, rect);
 	}
@@ -610,7 +810,7 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (xofs >= 0) {
 		r.x0 = xofs + scaled_width;
 		r.y0 = yofs;
-		r.x1 = width;
+		r.x1 = allocation.width;
 		r.y1 = yofs + scaled_height;
 		if (r.x0 < r.x1)
 			paint_background (view, &r, rect);
@@ -620,8 +820,8 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (yofs >= 0) {
 		r.x0 = 0;
 		r.y0 = yofs + scaled_height;
-		r.x1 = width;
-		r.y1 = height;
+		r.x1 = allocation.width;
+		r.y1 = allocation.height;
 		if (r.y0 < r.y1)
 			paint_background (view, &r, rect);
 	}
@@ -655,6 +855,12 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	eog_debug_message (DEBUG_WINDOW, "%s: x0: %i,\t y0: %i,\t x1: %i,\t y1: %i\n",
 			   str, d.x0, d.y0, d.x1, d.y1);
 
+#ifdef HAVE_RSVG
+	if (eog_image_is_svg (view->priv->image) && interp_type != GDK_INTERP_NEAREST) {
+		draw_svg (view, &d, &r);
+		return;
+	}
+#endif
 	/* Short-circuit the fast case to avoid a memcpy() */
 
 	if (is_unity_zoom (view)
@@ -670,8 +876,8 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 			  + (d.y0 - yofs) * rowstride
 			  + 3 * (d.x0 - xofs));
 
-		gdk_draw_rgb_image_dithalign (GTK_WIDGET (priv->display)->window,
-					      GTK_WIDGET (priv->display)->style->black_gc,
+		gdk_draw_rgb_image_dithalign (gtk_widget_get_window (GTK_WIDGET (priv->display)),
+					      gtk_widget_get_style (GTK_WIDGET (priv->display))->black_gc,
 					      d.x0, d.y0,
 					      d.x1 - d.x0, d.y1 - d.y0,
 					      GDK_RGB_DITHER_MAX,
@@ -692,29 +898,7 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	}
 
 	/* Compute transparency parameters */
-	switch (priv->transp_style) {
-	case EOG_TRANSP_BACKGROUND: {
-		GdkColor color = GTK_WIDGET (priv->display)->style->bg[GTK_STATE_NORMAL];
-
-		check_1 = check_2 = (((color.red & 0xff00) << 8)
-				     | (color.green & 0xff00)
-				     | ((color.blue & 0xff00) >> 8));
-		break; }
-
-	case EOG_TRANSP_CHECKED:
-		check_1 = CHECK_GRAY;
-		check_2 = CHECK_LIGHT;
-		break;
-
-	case EOG_TRANSP_COLOR:
-		check_1 = check_2 = priv->transp_color;
-		break;
-
-	default:
-		g_assert_not_reached ();
-	};
-
-	check_size = CHECK_MEDIUM;
+	get_transparency_params (view, &check_size, &check_1, &check_2);
 
 	/* Draw! */
 	gdk_pixbuf_composite_color (priv->pixbuf,
@@ -729,8 +913,8 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 				    check_size,
 				    check_1, check_2);
 
-	gdk_draw_rgb_image_dithalign (GTK_WIDGET (priv->display)->window,
-				      GTK_WIDGET (priv->display)->style->black_gc,
+	gdk_draw_rgb_image_dithalign (gtk_widget_get_window (priv->display),
+				      gtk_widget_get_style (priv->display)->black_gc,
 				      d.x0, d.y0,
 				      d.x1 - d.x0, d.y1 - d.y0,
 				      GDK_RGB_DITHER_MAX,
@@ -789,19 +973,21 @@ request_paint_area (EogScrollView *view, GdkRectangle *area)
 {
 	EogScrollViewPrivate *priv;
 	EogIRect r;
+	GtkAllocation allocation;
 
 	priv = view->priv;
 
 	eog_debug_message (DEBUG_WINDOW, "x: %i, y: %i, width: %i, height: %i\n",
 			   area->x, area->y, area->width, area->height);
 
-	if (!GTK_WIDGET_DRAWABLE (priv->display))
+	if (!gtk_widget_is_drawable (priv->display))
 		return;
 
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 	r.x0 = MAX (0, area->x);
 	r.y0 = MAX (0, area->y);
-	r.x1 = MIN (GTK_WIDGET (priv->display)->allocation.width, area->x + area->width);
-	r.y1 = MIN (GTK_WIDGET (priv->display)->allocation.height, area->y + area->height);
+	r.x1 = MIN (allocation.width, area->x + area->width);
+	r.y1 = MIN (allocation.height, area->y + area->height);
 
 	eog_debug_message (DEBUG_WINDOW, "r: %i, %i, %i, %i\n", r.x0, r.y0, r.x1, r.y1);
 
@@ -822,7 +1008,7 @@ request_paint_area (EogScrollView *view, GdkRectangle *area)
 		 * It's sufficient to add only a antitaliased idle update
 		 */
 		priv->progressive_state = PROGRESSIVE_NONE;
-	else if (!eog_image_is_animation (priv->image))
+	else if (!priv->image || !eog_image_is_animation (priv->image))
 		/* do nearest neigbor before anti aliased version,
 		   except for animations to avoid a "blinking" effect. */
 		paint_rectangle (view, &r, GDK_INTERP_NEAREST);
@@ -851,9 +1037,9 @@ static void
 scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	int xofs, yofs;
 	GdkWindow *window;
-	int width, height;
 	int src_x, src_y;
 	int dest_x, dest_y;
 	int twidth, theight;
@@ -861,14 +1047,16 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	priv = view->priv;
 
 	/* Check bounds & Compute offsets */
-	if (GTK_WIDGET_VISIBLE (priv->hbar)) {
-		x = CLAMP (x, 0, priv->hadj->upper - priv->hadj->page_size);
+	if (gtk_widget_get_visible (priv->hbar)) {
+		x = CLAMP (x, 0, gtk_adjustment_get_upper (priv->hadj)
+				 - gtk_adjustment_get_page_size (priv->hadj));
 		xofs = x - priv->xofs;
 	} else
 		xofs = 0;
 
-	if (GTK_WIDGET_VISIBLE (priv->vbar)) {
-		y = CLAMP (y, 0, priv->vadj->upper - priv->vadj->page_size);
+	if (gtk_widget_get_visible (priv->vbar)) {
+		y = CLAMP (y, 0, gtk_adjustment_get_upper (priv->vadj)
+				 - gtk_adjustment_get_page_size (priv->vadj));
 		yofs = y - priv->yofs;
 	} else
 		yofs = 0;
@@ -879,23 +1067,22 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	priv->xofs = x;
 	priv->yofs = y;
 
-	if (!GTK_WIDGET_DRAWABLE (priv->display))
+	if (!gtk_widget_is_drawable (priv->display))
 		goto out;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
-	if (abs (xofs) >= width || abs (yofs) >= height) {
+	if (abs (xofs) >= allocation.width || abs (yofs) >= allocation.height) {
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 		goto out;
 	}
 
-	window = GTK_WIDGET (priv->display)->window;
+	window = gtk_widget_get_window (GTK_WIDGET (priv->display));
 
 	/* Ensure that the uta has the full size */
 
-	twidth = (width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
-	theight = (height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
+	twidth = (allocation.width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
+	theight = (allocation.height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
 
 	if (priv->uta)
 		g_assert (priv->idle_id != 0);
@@ -915,7 +1102,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	uta_copy_area (priv->uta,
 		       src_x, src_y,
 		       dest_x, dest_y,
-		       width - abs (xofs), height - abs (yofs));
+		       allocation.width - abs (xofs),
+		       allocation.height - abs (yofs));
 
 	/* Scroll the window area and process exposure synchronously. */
 
@@ -933,11 +1121,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 		priv->vadj, G_SIGNAL_MATCH_DATA,
 		0, 0, NULL, NULL, view);
 
-	priv->hadj->value = x;
-	priv->vadj->value = y;
-
-	gtk_adjustment_value_changed (priv->hadj);
-	gtk_adjustment_value_changed (priv->vadj);
+	gtk_adjustment_set_value (priv->hadj, x);
+	gtk_adjustment_set_value (priv->vadj, y);
 
 	g_signal_handlers_unblock_matched (
 		priv->hadj, G_SIGNAL_MATCH_DATA,
@@ -971,7 +1156,8 @@ adjustment_changed_cb (GtkAdjustment *adj, gpointer data)
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
 
-	scroll_to (view, priv->hadj->value, priv->vadj->value, FALSE);
+	scroll_to (view, gtk_adjustment_get_value (priv->hadj),
+		   gtk_adjustment_get_value (priv->vadj), FALSE);
 }
 
 
@@ -1024,8 +1210,8 @@ set_zoom (EogScrollView *view, double zoom,
 	  gboolean have_anchor, int anchorx, int anchory)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	int xofs, yofs;
-	int disp_width, disp_height;
 	double x_rel, y_rel;
 
 	g_assert (zoom > 0.0);
@@ -1047,20 +1233,19 @@ set_zoom (EogScrollView *view, double zoom,
 
 	priv->zoom_mode = ZOOM_MODE_FREE;
 
-	disp_width = GTK_WIDGET (priv->display)->allocation.width;
-	disp_height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	/* compute new xofs/yofs values */
 	if (have_anchor) {
-		x_rel = (double) anchorx / disp_width;
-		y_rel = (double) anchory / disp_height;
+		x_rel = (double) anchorx / allocation.width;
+		y_rel = (double) anchory / allocation.height;
 	} else {
 		x_rel = 0.5;
 		y_rel = 0.5;
 	}
 
 	compute_center_zoom_offsets (view, priv->zoom, zoom,
-				     disp_width, disp_height,
+				     allocation.width, allocation.height,
 				     x_rel, y_rel,
 				     &xofs, &yofs);
 
@@ -1090,23 +1275,22 @@ static void
 set_zoom_fit (EogScrollView *view)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	double new_zoom;
-	int width, height;
 
 	priv = view->priv;
 
 	priv->zoom_mode = ZOOM_MODE_FIT;
 
-	if (!GTK_WIDGET_MAPPED (GTK_WIDGET (view)))
+	if (!gtk_widget_get_mapped (GTK_WIDGET (view)))
 		return;
 
 	if (priv->pixbuf == NULL)
 		return;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET(priv->display), &allocation);
 
-	new_zoom = zoom_fit_scale (width, height,
+	new_zoom = zoom_fit_scale (allocation.width, allocation.height,
 				   gdk_pixbuf_get_width (priv->pixbuf),
 				   gdk_pixbuf_get_height (priv->pixbuf),
 				   priv->upscale);
@@ -1135,11 +1319,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	EogScrollView *view;
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	gboolean do_zoom;
 	double zoom;
 	gboolean do_scroll;
 	int xofs, yofs;
-	int width, height;
 
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
@@ -1149,8 +1333,7 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	xofs = yofs = 0;
 	zoom = 1.0;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	/* EogScrollView doesn't handle/have any Alt+Key combos */
 	if (event->state & GDK_MOD1_MASK) {
@@ -1167,11 +1350,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_Page_Up:
 		do_scroll = TRUE;
 		if (event->state & GDK_CONTROL_MASK) {
-			xofs = -(width * 3) / 4;
+			xofs = -(allocation.width * 3) / 4;
 			yofs = 0;
 		} else {
 			xofs = 0;
-			yofs = -(height * 3) / 4;
+			yofs = -(allocation.height * 3) / 4;
 		}
 		break;
 
@@ -1184,11 +1367,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_Page_Down:
 		do_scroll = TRUE;
 		if (event->state & GDK_CONTROL_MASK) {
-			xofs = (width * 3) / 4;
+			xofs = (allocation.width * 3) / 4;
 			yofs = 0;
 		} else {
 			xofs = 0;
-			yofs = (height * 3) / 4;
+			yofs = (allocation.height * 3) / 4;
 		}
 		break;
 
@@ -1229,7 +1412,8 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	if (do_zoom) {
 		gint x, y;
 
-		gdk_window_get_pointer (widget->window, &x, &y, NULL);
+		gdk_window_get_pointer (gtk_widget_get_window (widget),
+					&x, &y, NULL);
 		set_zoom (view, zoom, TRUE, x, y);
 	}
 
@@ -1250,7 +1434,7 @@ eog_scroll_view_button_press_event (GtkWidget *widget, GdkEventButton *event, gp
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
 
-	if (!GTK_WIDGET_HAS_FOCUS (priv->display))
+	if (!gtk_widget_has_focus (priv->display))
 		gtk_widget_grab_focus (GTK_WIDGET (priv->display));
 
 	if (priv->dragging)
@@ -1341,9 +1525,9 @@ eog_scroll_view_scroll_event (GtkWidget *widget, GdkEventScroll *event, gpointer
 	priv = view->priv;
 
 	/* Compute zoom factor and scrolling offsets; we'll only use either of them */
-
-	xofs = priv->hadj->page_increment / 2; /* same as in gtkscrolledwindow.c */
-	yofs = priv->vadj->page_increment / 2;
+	/* same as in gtkscrolledwindow.c */
+	xofs = gtk_adjustment_get_page_increment (priv->hadj) / 2;
+	yofs = gtk_adjustment_get_page_increment (priv->vadj) / 2;
 
 	switch (event->direction) {
 	case GDK_SCROLL_UP:
@@ -1412,7 +1596,7 @@ eog_scroll_view_motion_event (GtkWidget *widget, GdkEventMotion *event, gpointer
 		return FALSE;
 
 	if (event->is_hint)
-		gdk_window_get_pointer (GTK_WIDGET (priv->display)->window, &x, &y, &mods);
+		gdk_window_get_pointer (gtk_widget_get_window (GTK_WIDGET (priv->display)), &x, &y, &mods);
 	else {
 		x = event->x;
 		y = event->y;
@@ -1931,8 +2115,8 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 
 	if (priv->image != NULL) {
 		free_image_resources (view);
-		if (GTK_WIDGET_DRAWABLE (priv->display) && image == NULL) {
-			gdk_window_clear (GTK_WIDGET (priv->display)->window);
+		if (gtk_widget_is_drawable (priv->display) && image == NULL) {
+			gdk_window_clear (gtk_widget_get_window (priv->display));
 		}
 	}
 	g_assert (priv->image == NULL);
@@ -1973,8 +2157,8 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 gboolean
 eog_scroll_view_scrollbars_visible (EogScrollView *view)
 {
-	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (view->priv->hbar)) &&
-	    !GTK_WIDGET_VISIBLE (GTK_WIDGET (view->priv->vbar)))
+	if (!gtk_widget_get_visible (GTK_WIDGET (view->priv->hbar)) &&
+	    !gtk_widget_get_visible (GTK_WIDGET (view->priv->vbar)))
 		return FALSE;
 
 	return TRUE;
@@ -2008,6 +2192,7 @@ eog_scroll_view_init (EogScrollView *view)
 	priv->transp_color = 0;
 	priv->cursor = EOG_SCROLL_VIEW_CURSOR_NORMAL;
 	priv->menu = NULL;
+	priv->background_surface = NULL;
 }
 
 static void
@@ -2031,6 +2216,11 @@ eog_scroll_view_dispose (GObject *object)
 		priv->idle_id = 0;
 	}
 
+	if (priv->background_surface != NULL) {
+		cairo_surface_destroy (priv->background_surface);
+		priv->background_surface = NULL;
+	}
+
 	free_image_resources (view);
 
 	G_OBJECT_CLASS (eog_scroll_view_parent_class)->dispose (object);
