@@ -42,6 +42,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 
 #if HAVE_EXEMPI
 #include <exempi/xmp.h>
@@ -57,6 +58,17 @@ static gboolean force_new_instance = FALSE;
 #endif
 static gchar **startup_files = NULL;
 
+static gboolean
+_print_version_and_exit (const gchar *option_name,
+			 const gchar *value,
+			 gpointer data,
+			 GError **error)
+{
+	g_print("%s %s\n", _("Eye of GNOME Image Viewer"), VERSION);
+	exit (EXIT_SUCCESS);
+	return TRUE;
+}
+
 static const GOptionEntry goption_options[] =
 {
 	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &fullscreen, N_("Open in fullscreen mode"), NULL  },
@@ -65,6 +77,8 @@ static const GOptionEntry goption_options[] =
 #if HAVE_DBUS
 	{ "new-instance", 'n', 0, G_OPTION_ARG_NONE, &force_new_instance, N_("Start a new instance instead of reusing an existing one"), NULL },
 #endif
+	{ "version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
+	  _print_version_and_exit, N_("Show the application's version"), NULL},
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &startup_files, NULL, N_("[FILE…]") },
 	{ NULL }
 };
@@ -217,6 +231,9 @@ main (int argc, char **argv)
 
 #ifdef HAVE_EXEMPI
  	xmp_init();
+#endif
+#ifdef HAVE_RSVG
+	rsvg_init();
 #endif
 	eog_debug_init ();
 	eog_job_queue_init ();
@@ -244,6 +261,9 @@ main (int argc, char **argv)
 
 	eog_plugin_engine_shutdown ();
 
+#ifdef HAVE_RSVG
+	rsvg_term();
+#endif
 #ifdef HAVE_EXEMPI
 	xmp_terminate();
 #endif
